
'=====================================================
'                                                    '
'        MouseOverControls.inc (Version 4)           '
'        ---------------------------------           '
'                                                    '
' This include code will let you know when the mouse ' 
' position is over a control that you specify.       '
'                                                    '
'                    By Andy.                        '
'                                                    '
'=====================================================


'=====================================================
' We need this function to get the mouse co-ordinates.
'=====================================================
$ifndef __windowssdk_inc__
    declare "user32",GetCursorPos(pPoint:POINTAPI),INT
    declare "user32",DestroyWindow(uint hWnd),int
$endif

Type CPos
     Def xposition:int
     Def yposition:int
Endtype

Def pt:Cpos

'========================================================
' Somewhere to store the control co-ordinates, sizes etc.
'========================================================
int ControlDown[10000]
int ControlAcross[10000]
int ControlWidth[10000]
int ControlHeight[10000]
int ControlNumber[10000]
int ControlOverNumber[10000]
UINT ControlsWindow[10000]
Uint WindowPointer[1000]
int WindowDown[1000]
int WindowAcross[1000]
int WindowWidth[1000]
int WindowHeight[1000]
int WindowOverNumber[1000]
string WindowName[1000]
string WindowDesc[1000]
string ControlType[10000]
string MouseOverType = ""

'=====================
' Some variables used.
'=====================
int ControlIndex,WindowIndex,ControlXpos,ControlYpos,WindowXpos,WindowYpos
int ycounter,zcounter,wcounter,wmcounter
int ControlUpdated,WindowUpdated
int MouseIsOver         = 0
int NoLongerOver        = 0
int MouseIsOverWindow   = 0
int MouseIsOverControl  = 0
int WindowExistsCheck   = 0
int MinTest             = 0
int MaxTest             = 0
int NormTest            = 0
int WindowStateTest     = 0
int WindowMin           = 0
int WindowMovedCheck    = 0
int OldWindowAcross     = 0
int OldWindowDown       = 0
int OldWindowWidth      = 0
int OldWindowHeight     = 0
uint OldWindow          = 0

'============================================================
' Here we store a window's name after the OPENWINDOW command. 
'============================================================
sub SetWindowName(window WindowIn,string WindowNameIn),int
    SetWindowLong(WindowIn.hwnd, GWL_USERDATA, &WindowNameIn)
return 0
endsub

'============================================================================================
' Store the control co-ordinates, but first check to see if the control for that window
' has already been stored - if so update the handle of the window for that control, otherwise
' it has not been stored before - so store it.
'============================================================================================
sub MouseOverControl(window MyWin,int MyControl)

	 string MyWinDescX = ""
	 pointer prop = GetWindowLong(MyWin.hwnd, GWL_USERDATA)		
	 IF prop THEN MyWinDescX = *<string>prop
	 delete prop

	 WindowForMouse(MyWin,MyWinDescX)

	 ControlUpdated = 0

	 '================================================================================
	 ' If control details have already been stored - update the window handle details.
	 '
	 ' Closing a window and then re-opening it again causes that window to change It's
	 ' handle details, so always update each control with the current window handle.
	 '================================================================================
	 if ControlIndex >= 1
		 for zcounter = 1 to ControlIndex
			  if ControlNumber[zcounter] = MyControl and WindowName[zcounter] = ltrim$(MyWinDescX)
				  ControlsWindow[zcounter] = MyWin.hwnd
				  ControlUpdated = 1
				  breakfor
			  endif
		 next zcounter
	 endif

	 '==================================================================
	 ' Control details have not yet been stored - so here we store them.
	 '==================================================================
	 if ControlUpdated = 0
		 ControlIndex += 1
		 GetSize MyWin,ControlAcross[ControlIndex],ControlDown[ControlIndex],ControlWidth[ControlIndex],ControlHeight[ControlIndex],MyControl
		 ControlNumber[ControlIndex] = MyControl
		 ControlsWindow[ControlIndex] = MyWin.hwnd
		 WindowName[ControlIndex]   = ltrim$(MyWinDescX)
		 ControlOverNumber[ControlIndex] = 0
	 endif

return 0
endsub

'=============================================================================================
' Here we check to see if the mouse is over a control - 0 false, 1 Moved over, 2 Remains over. 
'=============================================================================================
sub MouseOver(window ControlsWindowIn,INT ButtonIndex),int

    GetCursorPos(pt)
    ControlXpos = pt.xposition
    ControlYpos = pt.yposition

    MouseIsOverControl = 0
    MouseOverType = ""

    for zcounter = 1 to ControlIndex
        if ControlNumber[zcounter] = ButtonIndex and ControlsWindow[zcounter] = ControlsWindowIn.hwnd
           if ControlXpos >= ControlAcross[zcounter] and ControlXpos <= ControlAcross[zcounter] + ControlWidth[zcounter] and ControlYpos >= ControlDown[zcounter] and ControlYpos <= ControlDown[zcounter] + ControlHeight[zcounter] 

              if ControlType[zcounter] = ""

                  string controlName = ""
						POINT WindowInp
						WindowInp.x = MOUSEX()
						WindowInp.y = MOUSEY()

						ScreenToClient(ControlsWindowIn.hwnd,WindowInp)
						
						HWND handle = ChildWindowFromPoint(ControlsWindowIn.hwnd,WindowInp)
						
						IF handle THEN
							GetClassName(handle, controlName, 255)
							
							' Here we handle the different button types
							IF UCASE$(controlName) = UCASE$("Button") THEN
								DWORD dwStyle = GetWindowLong(handle, GWL_STYLE)
								
								IF (dwStyle & BS_TYPEMASK) = BS_AUTOCHECKBOX THEN controlName = "CHECKBOX"
								IF (dwStyle & BS_TYPEMASK) = BS_AUTORADIOBUTTON THEN controlName = "RADIOBUTTON"
							ENDIF
					
						ENDIF

						ControlType[zcounter] = ucase$(controlName) 

              endif

              if ControlOverNumber[zcounter] = 0
                 MouseIsOverControl = 1
                 ControlOverNumber[zcounter] = 1 
                 MouseOverType = ControlType[zcounter]
              else
                 MouseIsOverControl = 2
                 MouseOverType = ControlType[zcounter]
              endif
           else
              MouseIsOverControl = 0
              MouseOverType = ""
           endif
           breakfor
        endif
    next zcounter

return MouseIsOverControl
endsub

'===============================================================================================
' Here we check if control was pointed to but is no longer pointed at - Returns 1 true, 0 false.  
'===============================================================================================
sub MouseMovedOff(window ControlsWindowIn,INT MovedOff),int

    NoLongerOver = 0

    for ycounter = 1 to ControlIndex
        if ControlNumber[ycounter] = MovedOff and ControlsWindow[ycounter] = ControlsWindowIn.hwnd
           if ControlOverNumber[ycounter] = 1 and MouseIsOverControl <> 2
              ControlOverNumber[ycounter] = 0
              NoLongerOver = 1 
           endif
           breakfor
        endif
    next ycounter

return NoLongerOver
endsub

'===========================================================================
' Store the window co-ordinates, but first check to see if the window
' has already been stored - if so update the handle of the window, otherwise
' it has not been stored before - so store it.
'===========================================================================
sub WindowForMouse(window MyWin,string MyWinDesc)

WindowUpdated = 0

'================================================================================
' If window details have already been stored - update the window handle details.
'
' Closing a window and then re-opening it again causes that window to change It's
' handle details, so always update each control with the current window handle.
'================================================================================
if WindowIndex >= 1
   for wcounter = 1 to WindowIndex
       if WindowDesc[wcounter] = ltrim$(MyWinDesc)
          WindowPointer[wcounter] = MyWin.hwnd
          WindowUpdated = 1
          breakfor
       endif
   next wcounter
endif

'=================================================================
' Window details have not yet been stored - so here we store them.
'=================================================================
if WindowUpdated = 0
   WindowIndex += 1
   GetSize MyWin,WindowAcross[WindowIndex],WindowDown[WindowIndex],WindowWidth[WindowIndex],WindowHeight[WindowIndex]
   WindowPointer[WindowIndex] = MyWin.hwnd
   WindowDesc[WindowIndex]   = ltrim$(MyWinDesc)
   WindowOverNumber[WindowIndex] = 0
endif

GetSize MyWin,OldWindowAcross,OldWindowDown,OldWindowWidth,OldWindowHeight
OldWindow = MyWin.hwnd

return 0
endsub

'==============================================================================
' Here we check to see if the mouse is over the window - 0 false, 1 Moved over. 
'==============================================================================
sub TrackWindow(window WindowIn),int

	 string MyWinDescX = ""
	 pointer prop = GetWindowLong(WindowIn.hwnd, GWL_USERDATA)		
	 IF prop THEN MyWinDescX = *<string>prop
	 delete prop

	 GetCursorPos(pt)
	 WindowXpos = pt.xposition
	 WindowYpos = pt.yposition

	 for wcounter = 1 to WindowIndex
	     if WindowPointer[wcounter] = WindowIn.hwnd and WindowDesc[wcounter] = ltrim$(MyWinDescX)

		     '=============================================
		     ' Update the window with it's new coordinates.
		     '=============================================

		     GetSize WindowIn,WindowAcross[wcounter],WindowDown[wcounter],WindowWidth[wcounter],WindowHeight[wcounter]

           if OldWindow = WindowIn.hwnd and OldWindowAcross <> WindowAcross[wcounter] or OldWindowDown <> WindowDown[wcounter] or OldWindowWidth <> WindowWidth[wcounter] or OldWindowHeight <> WindowHeight[wcounter]
			     GetSize WindowIn,OldWindowAcross,OldWindowDown,OldWindowWidth,OldWindowHeight
              WindowMovedCheck = 1
           else
              WindowMovedCheck = 0
           endif 

		     '======================================================
		     ' Here we check to see if the mouse is over the window.
		     '======================================================

		     if WindowXpos >= WindowAcross[wcounter] and WindowXpos <= WindowAcross[wcounter] + WindowWidth[wcounter] and WindowYpos >= WindowDown[wcounter] and WindowYpos <= WindowDown[wcounter] + WindowHeight[wcounter] 
			     MouseIsOverWindow = 1
		     else
			     MouseIsOverWindow = 0
		     endif
		     breakfor
	     endif
	 next wcounter

    '===========================================================
    ' Update the controls with their new co-ordinates and sizes. 
    '===========================================================

	 for wmcounter = 1 to ControlIndex

 		  if ControlsWindow[wmcounter] = WindowIn.hwnd and WindowName[wmcounter] = ltrim$(MyWinDescX)
           getsize WindowIn,ControlAcross[wmcounter],ControlDown[wmcounter],ControlWidth[wmcounter],ControlHeight[wmcounter],ControlNumber[wmcounter]
		  endif

	 next wmcounter

return MouseIsOverWindow
endsub


'=============================================
' Removes a control from the specified window.
'=============================================
sub DeleteControl(window WindowIn,int ControlNumberIn,string MyWinDesc),int
    IF CONTROLEXISTs(WindowIn,ControlNumberIn)
       DestroyWindow(GETCONTROLHANDLE(WindowIn,ControlNumberIn))
		 for zcounter = 1 to ControlIndex
			  if ControlNumber[zcounter] = ControlNumberIn and WindowName[zcounter] = ltrim$(MyWinDesc)
				  ControlNumber[zcounter] = 10000
				  breakfor
			  endif
		 next zcounter
       Return 1
    endif
return 0
endsub


'================================================
' Creates a Button and registers it for tracking.
'================================================
sub CreateButton(window WindowIn,string ButtonTextIn,int ButAcross,int ButDown,int ButLength,int ButHeight,int ButtonNumber)
	 IF !CONTROLEXISTs(WindowIn,ButtonNumber)
		 uint dwStyle = WS_VISIBLE | WS_CHILD | WS_TABSTOP
		 CreateWindowEx(0, "BUTTON", ButtonTextIn, dwStyle | BS_PUSHBUTTON,ButAcross,ButDown,ButLength,ButHeight,WindowIn.hwnd,ButtonNumber,GetModuleHandle(0), 0)
		 MouseOverControl(WindowIn,ButtonNumber)
	 endif
return
endsub


'==================================================
' Creates a Checkbox and registers it for tracking.
'==================================================
sub CreateCheckBox(window WindowIn,string ButtonTextIn,int ButAcross,int ButDown,int ButLength,int ButHeight,int ButtonNumber)
	 IF !CONTROLEXISTs(WindowIn,ButtonNumber)
		 uint dwStyle = WS_VISIBLE | WS_CHILD | WS_TABSTOP
		 CreateWindowEx(0, "BUTTON", ButtonTextIn, dwStyle | BS_AUTOCHECKBOX,ButAcross,ButDown,ButLength,ButHeight,WindowIn.hwnd,ButtonNumber,GetModuleHandle(0), 0)
		 MouseOverControl(WindowIn,ButtonNumber)
	 endif
return
endsub


'=====================================================
' Creates a Radiobutton and registers it for tracking.
'=====================================================
sub CreateRadio(window WindowIn,string ButtonTextIn,int ButAcross,int ButDown,int ButLength,int ButHeight,int ButtonNumber)
	 IF !CONTROLEXISTs(WindowIn,ButtonNumber)
		 uint dwStyle = WS_VISIBLE | WS_CHILD | WS_TABSTOP
		 CreateWindowEx(0, "BUTTON", ButtonTextIn, dwStyle | BS_AUTORADIOBUTTON,ButAcross,ButDown,ButLength,ButHeight,WindowIn.hwnd,ButtonNumber,GetModuleHandle(0), 0)
		 MouseOverControl(WindowIn,ButtonNumber)
	 endif
return
endsub


'=======================================================================================
' Here we check to see if the mouse is over any specific control type - 0 false, 1 true. 
'=======================================================================================
SUB MouseOverControlType(WINDOW WindowIn,string ControlTypeIn),int 
	 string controlName = ""
	 POINT mop
	 mop.x = MOUSEX()
	 mop.y = MOUSEY()
	 ScreenToClient(WindowIn.hwnd,mop)
	
	 HWND handle = ChildWindowFromPoint(WindowIn.hwnd,mop)
	
	 IF handle THEN
		 GetClassName(handle, controlName, 255)
		
		 ' Here we handle the different button types
		 IF UCASE$(controlName) = UCASE$("Button") THEN
			 DWORD dwStyle = GetWindowLong(handle, GWL_STYLE)
			 IF (dwStyle & BS_TYPEMASK) = BS_AUTOCHECKBOX THEN controlName = "CHECKBOX"
			 IF (dwStyle & BS_TYPEMASK) = BS_AUTORADIOBUTTON THEN controlName = "RADIOBUTTON"
		 ENDIF
	 ENDIF

    ' B - Button
    ' C - Checkbox
    ' E - Edit
    ' L - Listbox
    ' R - Radio
    ' S - Static
    ' V - Listview

    if ucase$(controlName) = "BUTTON"      and ucase$(ControlTypeIn) = "B" then return 1 
    if ucase$(controlName) = "CHECKBOX"    and ucase$(ControlTypeIn) = "C" then return 1 
    if ucase$(controlName) = "EDIT"        and ucase$(ControlTypeIn) = "E" then return 1 
    if ucase$(controlName) = "LISTBOX"     and ucase$(ControlTypeIn) = "L" then return 1 
    if ucase$(controlName) = "RADIOBUTTON" and ucase$(ControlTypeIn) = "R" then return 1 
    if ucase$(controlName) = "STATIC"      and ucase$(ControlTypeIn) = "S" then return 1 
    if ucase$(controlName) = "LISTVIEW"    and ucase$(ControlTypeIn) = "V" then return 1 
	
	 RETURN 0
ENDSUB


'=========================================================================
' Here we check to see if the mouse is over any control - 0 false, 1 true. 
'=========================================================================
SUB MouseOverAnyControl(WINDOW WindowIn),int 
	 string controlName = ""
	 POINT WindowInp
	 WindowInp.x = MOUSEX()
	 WindowInp.y = MOUSEY()
	 ScreenToClient(WindowIn.hwnd,WindowInp)
	
	 HWND handle = ChildWindowFromPoint(WindowIn.hwnd,WindowInp)
	
	 IF handle THEN
		 GetClassName(handle, controlName, 255)
		
		 ' Here we handle the different button types
		 IF UCASE$(controlName) = UCASE$("Button") THEN
			 DWORD dwStyle = GetWindowLong(handle, GWL_STYLE)
			
			 IF (dwStyle & BS_TYPEMASK) = BS_AUTOCHECKBOX THEN controlName = "CHECKBOX"
			 IF (dwStyle & BS_TYPEMASK) = BS_AUTORADIOBUTTON THEN controlName = "RADIOBUTTON"
		 ENDIF

	 ENDIF

    if ucase$(controlName) = "BUTTON"      then return 1 
    if ucase$(controlName) = "CHECKBOX"    then return 1 
    if ucase$(controlName) = "EDIT"        then return 1 
    if ucase$(controlName) = "LISTBOX"     then return 1 
    if ucase$(controlName) = "RADIOBUTTON" then return 1 
    if ucase$(controlName) = "STATIC"      Then return 1 
    if ucase$(controlName) = "LISTVIEW"    then return 1

return 0
ENDSUB

'==============================================================
' Here we check to see what type of control the mouse is over.
'
' Return values (A string in uppercase):
'
' "BUTTON"       
' "CHECKBOX"     
' "EDIT"        
' "LISTBOX"     
' "RADIOBUTTON"  
' "STATIC"      
' "LISTVIEW" 
' "CAPTION"  
' "EMERGENCEWNDCLASS" - over the window, but not over a control
' "" - Not over the window.
'
'==============================================================
SUB MouseOverWhichType(WINDOW WindowIn),string 
	 string controlName = ""
	 POINT WindowInp
	 WindowInp.x = MOUSEX()
	 WindowInp.y = MOUSEY()
	 ScreenToClient(WindowIn.hwnd,WindowInp)
	
	 HWND handle = ChildWindowFromPoint(WindowIn.hwnd,WindowInp)
	
	 IF handle THEN
	 	 GetClassName(handle, controlName, 255)
		
		 ' Here we handle the different button types
		 IF UCASE$(controlName) = UCASE$("Button") THEN
			 DWORD dwStyle = GetWindowLong(handle, GWL_STYLE)
			 IF (dwStyle & BS_TYPEMASK) = BS_AUTOCHECKBOX THEN controlName = "CHECKBOX"
			 IF (dwStyle & BS_TYPEMASK) = BS_AUTORADIOBUTTON THEN controlName = "RADIOBUTTON"
		 ENDIF
 
	 ENDIF

    int Trw = TrackWindow(WindowIn)

	 if Trw = 1 and controlName = ""
		 controlName = "Caption"
	 endif 

return ucase$(controlName)
ENDSUB

'==============================================================================
' Here we check to see if the mouse is over the caption area - 0 false, 1 true. 
'==============================================================================
SUB MouseOverCaption(WINDOW WindowIn),int 
	 string controlName = ""
	 POINT WindowInp
	 WindowInp.x = MOUSEX()
	 WindowInp.y = MOUSEY()
	 ScreenToClient(WindowIn.hwnd,WindowInp)
	
	 HWND handle = ChildWindowFromPoint(WindowIn.hwnd,WindowInp)
	
	 IF handle THEN
	 	 GetClassName(handle, controlName, 255)
		
		 ' Here we handle the different button types
		 IF UCASE$(controlName) = UCASE$("Button") THEN
			 DWORD dwStyle = GetWindowLong(handle, GWL_STYLE)
			 IF (dwStyle & BS_TYPEMASK) = BS_AUTOCHECKBOX THEN controlName = "CHECKBOX"
			 IF (dwStyle & BS_TYPEMASK) = BS_AUTORADIOBUTTON THEN controlName = "RADIOBUTTON"
		 ENDIF
 
	 ENDIF

    int Trw = TrackWindow(WindowIn)

	 if Trw = 1 and controlName = ""
		 return 1
	 endif 

return 0
ENDSUB

'===============================================
' You can use this to see if a window has focus. 
'===============================================
sub WindowHasFocus(window UsingWindow),int 
    if UsingWindow = GetForegroundWindow()
       return 1
    endif

return 0
endsub

'===============================================================================
' You can use this to see if a window is minimised / maximised or normal window. 
'===============================================================================
sub WindowMinimised(window MyWindowMin),int
	 if IsWindowVisible(MyWindowMin.hwnd)
		 if IsIconic(MyWindowMin.hwnd) 
          return 1 
		endif
    endif
return 0
endsub

sub WindowMinimized(window MyWindowMin),int
	 if IsWindowVisible(MyWindowMin.hwnd)
		 if IsIconic(MyWindowMin.hwnd) 
          return 1 
		endif
    endif
return 0
endsub

sub WindowMaximised(window MyWindowMax),int
	 if IsWindowVisible(MyWindowMax.hwnd)
		 if IsZoomed(MyWindowMax.hwnd) 
          return 1 
		endif
    endif
return 0
endsub

sub WindowMaximized(window MyWindowMax),int
	 if IsWindowVisible(MyWindowMax.hwnd)
		 if IsZoomed(MyWindowMax.hwnd) 
          return 1 
		endif
    endif
return 0
endsub

sub WindowNormal(window MyWindowNorm),int
	 if IsWindowVisible(MyWindowNorm.hwnd)
		 if !IsIconic(MyWindowNorm.hwnd)
			 if !IsZoomed(MyWindowNorm.hwnd) 
             return 1
          endif 
		endif
    endif
return 0
endsub

sub WindowMoved(window MyWindowIn),int 
return WindowMovedCheck
endsub

SUB StartTracking(WINDOW WindowIn,opt int MyTimerSpeed,opt int MyTimerID),int 
	 string MyWinDescX = ""
	 pointer prop = GetWindowLong(WindowIn.hwnd, GWL_USERDATA)		
	 IF prop THEN MyWinDescX = *<string>prop
	 delete prop

    WindowMovedCheck = 0

    if MyTimerSpeed < 1
       MyTimerSpeed = 50
    endif

    if MyTimerID > 0
       starttimer WindowIn,MyTimerSpeed,MyTimerID
    else
       starttimer WindowIn,MyTimerSpeed
    endif

return 0
endsub

SUB StopTracking(WINDOW WindowIn,opt int MyTimerID),int 
	 string MyWinDescX = ""
	 pointer prop = GetWindowLong(WindowIn.hwnd, GWL_USERDATA)		
	 IF prop THEN MyWinDescX = *<string>prop
	 delete prop

    WindowMovedCheck = 0

    if MyTimerID > 0
       stoptimer WindowIn,MyTimerID
    else
       stoptimer WindowIn
    endif

return 0
endsub

'=============================
' End of MouseOverControls.inc
'=============================


